/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;

/**
 * A comparator does comparisons of objects in the universe the comparator
 * is defined over.
 * A comparator throws an InvalidElementException if passed a null element, or
 * an element outside of its universe.
 * 
 * @author Mark Handy
 * @version JSDL-Teach release 1.0
 */

public interface Comparator {

/** 
 * Tests equality of two objects in the universe over which this
 * comparator is defined.
 * @param a 
 * @param b
 * @return whether a is equal to b
 */
    public boolean isEqualTo (Object a, Object b)
	throws InvalidElementException;


/** 
 * Tests strict order of two objects in the universe over which this
 * comparator is defined.
 * @param a
 * @param b
 * @return whether a is greater than b
 */
    public boolean isGreaterThan (Object a, Object b)
	throws InvalidElementException;

    
/** 
 * Tests strict order of two objects in the universe over which this
 * comparator is defined.
 * @param a
 * @param b
 * @return whether a is less than b
 */
    public boolean isLessThan (Object a, Object b)
	throws InvalidElementException;

    
/** 
 * Tests non-strict order of two objects in the universe over which this
 * comparator is defined.
 * @param a
 * @param b
 * @return whether a is greater than or equal to b
 */
    public boolean isGreaterThanOrEqualTo (Object a, Object b)
	throws InvalidElementException;

    
/** 
 * Tests non-strict order of two objects in the universe over which this
 * comparator is defined.
 * @param a
 * @param b
 * @return whether a is less than or equal to b
 */
    public boolean isLessThanOrEqualTo (Object a, Object b)
	throws InvalidElementException;

/** 
 * Tests whether this comparator can compare this object
 * @param a
 * @return whether a can be compared to other objects using this comparator
 */
    public boolean isComparable (Object a);

}
