/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;

/**
 * Interface to support containers that rely on an underlying ordering
 * of the elements.  Since the PriorityQueue and Dictionary interfaces
 * are not written to force use of comparators, containers that do use
 * them need to implement this interface, as well.<p>
 * Any class implementing ComparatorBased should provide require that the class
 * be constructed with a Comparator (for example, as a parameter to the
 * constructor), and should not allow the class to be created without one (for
 * example, no zero argument constructor).  Any class allowing itself to be
 * constructed without a comparator should throw an
 * <code>InvalidComparatorException</code> from any method that fails because a
 * comparator has not been set.  
 * @version JSDL-Teach release 1.0
 * @author Mark Handy (mdh)
 * @author Mike Boilen (mgb)
 * @see InvalidComparatorException
 */

public interface ComparatorBased {

/** 
 * This method establishes a ContainerInterfaces.Comparator that an
 * ordered container should use to compare its elements.  It must be
 * called after a container is instantiated.  It should not be called on a
 * non-empty container.
 * 
 * @param c A <code>jdsl.core.api.Comparator</code> appropriate to the
 * elements stored by the container
 * @return The old Comparator
 * @exception NotEmptyContainerException if the container has elements in it
 * 		when <code>setComparator(.)</code> is called.
 */
  public Comparator setComparator(Comparator c) throws NotEmptyContainerException;

/** 
	* Returns the <code>Comparator</code> used by this
	* <code>ComparatorBased</code> container.
 * 
 * @param c A <code>jdsl.core.api.Comparator</code> appropriate to the
 * elements stored by the container
 * @return The old Comparator
 */
  public Comparator comparator();
  
}
