/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;
import java.util.Enumeration;

/**
 * A container is a collection of elements, each of which is a generic
 * Object.  An element can be stored multiple times in a container.
 * <p> This is the base interface for all the container interfaces in
 * the "core" package (e.g., <code>Sequence,</code>
 * <code>PriorityQueue),</code> by way of the subinterfaces
 * <code>PositionalContainer</code> and <code>KeyBasedContainer.</code>
 *
 * @version JSDL-Teach release 1.0
 * @author Michael T. Goodrich
 * @author Mark Handy
 * @author Roberto Tamassia
 * @see PositionalContainer
 * @see KeyBasedContainer
 * @see jdsl.simple.api.Container
 */

public interface Container extends jdsl.simple.api.Container {
    
/**
 * Returns an enumeration of the elements stored in the container.
 * Duplicated elements appear in the enumeration as many times as they
 * are held in the container.  For some containers, the order of
 * the elements in the enumeration is arbitrary, but for some it is
 * defined.
 * 
 * @return Enumeration of all elements in the container
 */
  public Enumeration elements();
  
/**
 *  Instantiates another container of the same class (without knowing
 * the class).
 * 
 * @return a new instance of the class of the container on which the
 * method is called
 */
  public Container newContainer()
    throws InvalidContainerException;

}
  
