/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;
import java.util.Enumeration;

/**
  * Interface that defines the properties of an InspectableTree
  *
 * @author Mike Boilen (mgb)
  * @version JSDL-Teach release 1.0
  */

public interface InspectableTree extends PositionalContainer {

  

  /* *********************  Query Methods  ********************* */


/** 
 * 
 * @param p Any node of the tree
 * @return Whether the given node is the root of the tree
 * @exception InvalidPositionException if p is null or is incompatible with 
 * this container
 * @exception InvalidContainerException if this container is invalid
 */
  public boolean isRoot (Position p) throws InvalidPositionException,InvalidContainerException;


/** 
 * 
 * @param p Any node of the tree
 * @return Whether the given node has at least one child
 * @exception InvalidPositionException if p is null or is incompatible with 
 * this container
 * @exception InvalidContainerException if this container is invalid
 */
  public boolean isInternal (Position p) throws InvalidPositionException,InvalidContainerException;


/** 
 * 
 * @param p Any node of the tree
 * @return Whether the given node has zero children
 * @exception InvalidPositionException if p is null or is incompatible with 
 * this container
 * @exception InvalidContainerException if this container is invalid
 */
  public boolean isExternal (Position p) throws InvalidPositionException,InvalidContainerException;


  

  /* *********************  Accessor Methods  ********************* */


/** 
 * Note that trees always have at least one external node, so they always
 * have a root.
 * @return The top node of the tree
 * @exception InvalidContainerException if this container is invalid
 */
  public Position root()throws InvalidContainerException;

  
/** 
 * 
 * @param p Any node of the tree
 * @return Parent Position of the given node
 * @exception BoundaryViolationException If p is the root of the tree
 * @exception InvalidPositionException if p is null or is incompatible with 
 * this container
 * @exception InvalidContainerException if this container is invalid
 */
  public Position parent (Position p) throws
    InvalidPositionException, BoundaryViolationException,InvalidContainerException;


/** 
 * The returned Enumeration is guaranteed to give the children in order from left to right.
 * @param p Any node of the tree
 * @return Enumeration of all the children of that node
 * @exception InvalidPositionException if p is null or is incompatible with 
 * this container
 * @exception InvalidContainerException if this container is invalid
 */
  public Enumeration children (Position p) throws InvalidPositionException,InvalidContainerException;


/** 
 * The returned Enumeration is guaranteed to give the siblings in order from left to right.
 * @param p Any node of the tree
 * @return Enumeration of all the other children of the same parent
 * @exception InvalidPositionException if p is null or is incompatible with 
 * this container
 * @exception InvalidContainerException if this container is invalid
 */
  public Enumeration siblings (Position p) throws InvalidPositionException,InvalidContainerException;
  
}
