/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;
import java.util.Enumeration;

/**

 * A positional container stores elements at its positions and defines
 * adjacency relationships between the positions (for example,
 * before/after in a sequence, parent/child in a tree).  This is the
 * base interface for all the positional containers (e.g., Sequence,
 * Tree, Graph).
 *
 * @author Michael T. Goodrich
 * @author Mark Handy
 * @author Roberto Tamassia
 * @author Mike Boilen (mgb)
 * @version JSDL-Teach release 1.0
 * @see Position
 * @see Container
 * @see Sequence
 * @see Tree
 * @see BinaryTree
 * @see Graph
 */

public interface PositionalContainer extends Container {
 
/** 
 * Return an <code>Enumeration</code> of the positions in the container.  There
 * is no guarantee of the order of the positions in the
 * <code>Enumeration.</code> However, certain implementations of
 * <code>PositionalContainer</code> may guarantee a certain order of the
 * positions in the <code>Enumeration.</code>
 * 
 * @return <code>Enumeration</code> of all positions in the container
 */
  public Enumeration positions();

/** 
 * Replace the element at the specified position with a new element.
 *
 * @param p position at which the replacement is to occur
 * @param newElement new element to be stored at position p
 * @return old element formerly stored at position p
 * @exception InvalidPositionException is thrown if the specified position
 * does not belong to the container or if the position is <code>null</code>.
 */
  public Object replace (Position p, Object newElement)
    throws InvalidPositionException;

/**
	* Swaps the elements at two specified positions.  
 *
 * @param p first position participating in the swap.
 * @param q second position participating in the swap.  
 * @exception InvalidPositionException if thrown if on or both of the specified
 * <code>Positions</code> does not belong to the same implementation of
 * <code>Container,</code> or if either <code>Position</code> is
 * <code>null.</code>
 */
  public void swap (Position p, Position q)
    throws InvalidPositionException, InvalidContainerException;

}
