/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.simple.api;
import jdsl.core.api.EmptyContainerException;
import jdsl.core.api.InvalidKeyException;

/** 

 * Interface for a priority queue. <p> A priority queue defines a
 * total order relation, usually induced by a comparator.  A priority
 * queue stores a collection of items (each of which is a key-element
 * pair); it supports the insertion of items whose key in the domain
 * of the total order relation, and the access and removal of items
 * with smallest key.  <p> This interface in the "simple" package
 * defines only the basic methods of a priority queue.
 * @author Michael T. Goodrich
 * @author Mark Handy
 * @author Roberto Tamassia
 * @version JSDL-Teach release 1.0
 * @see EmptyContainerException
 * @see InvalidKeyException
 * @see jdsl.core.api.Comparator
 * @see jdsl.core.api.PriorityQueue
 */

public interface SimplePriorityQueue extends Container {
  
/** 
 * Inspect an element (not a key) with smallest key, without modifying the
 * priority queue.
 *
 * @return element with smallest key.
 * @exception EmptyContainerException if the container is empty.
 */
  public Object minElement()
    throws EmptyContainerException;

/** 
 * Inspect a smallest key (not its element), without modifying the
 * priority queue.
 *
 * @return a smallest key.
 * @exception EmptyContainerException if the container is empty.
 */
  public Object minKey()
    throws EmptyContainerException;
  
/** 
 * Insert an item (key-element pair).
 * @param key object.
 * @param element arbitrary object.
 * @exception InvalidKeyException is thrown if the key is not in the domain of
 * the total order relation.
 */
  public void insertItem(Object key, Object element)
    throws InvalidKeyException;
  
/** 
 * Remove an item (key,element) with smallest key.
 *
 * @return element (not the key) of the removed item.
 * @exception InvalidKeyException is thrown if the key is not in the domain of
 * the total order relation.
 */
  public Object removeMinElement()
    throws EmptyContainerException;
  
}
